/*

 * @Author: 祝玲云
 * @Date: 2021-05-20 13:28:24
 * @LastEditTime: 2021-05-21 18:15:12
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: \xs-plugins\src\lib\menu.js
 */

const fs = require("fs");
const path = require("path");
const compiler = require('vue-template-compiler')
let uploadApiList = []; // 最终生成api文件依赖数组
let fileContent = ""; // 所有带api映射关系的文件
let baseApi = {}; // 路由中的api映
let api = "";
class analysisPlugin {
  constructor(options) {
    this.options = options;
    this.options.entry = path.join(options.rootPath, '/' + this.options.entry)
    api = this.options.api;
    this.filePath = path.join(this.options.rootPath, "/src/modules");
  }
  apply() {
    this.getApiList(this.filePath);
    uploadApiList = this.getNodeTree(
      this.options.router.filter((item) => item.alias)
    );
    let data = {
      version: new Date().getTime(),
      list: uploadApiList,
    };
    let outputPath = path.join(
      this.options.rootPath,
      "/" + this.options.autoPath + "/menu.js"
    );
    fs.writeFile(
      outputPath,
      "export default " + JSON.stringify(data),
      (err) => {
        console.log(err);
      }
    );
  }

  // 根据路由遍历文件
  getNodeTree(routerList, parent) {
    const allRouter = [];
    routerList.filter((item) => {
      // 路由地址
      let fullPath = (parent && parent.url ? parent.url + "/" : "") + item.path;
      fullPath = fullPath.startsWith("/") ? fullPath : "/" + fullPath;
      item["url"] = fullPath;
      //单条ID
      let apiList = []; // api 映射数组
      let obj = {
        title: "",
        icon: "",
        url: fullPath,
        hide: false,
        apis: [],
        children: item.children ? this.getNodeTree(item.children, item) : [],
      };
      if (item.meta && item.name !== "child-frame") {
        if (item.meta) {
          obj.hide = item.meta.hidden ? item.meta.hidden : false;
          obj.icon = item.meta.icon ? item.meta.icon : "";
          obj.title = item.meta.title ? item.meta.title : "";
        }
        if (item.meta.moduleName) {
          obj["moduleName"] = item.meta.moduleName;
        }
        // 获取 $$router 中的 api
        if (item.meta.api) {
          apiList = apiList.concat(
            this.getApiByRouter(
              item.meta.api ? Array.from(new Set(item.meta.api)) : []
            )
          );
        }
        //  获取 script 中的 api, 除了$$router部分
        let componentDevUrl = this.formatPath(item.components.default);
        if (componentDevUrl) {
          let index = componentDevUrl.indexOf( "/" + this.options.moduleConfig.CUR_MODULES + "/");

          let componentUrl = path.join(this.options.entry, componentDevUrl.substr(index + this.options.moduleConfig.CUR_MODULES.length + 1)).replace(/[\\]{1}/g, "/");
          const content = fs.readFileSync(componentUrl, "utf-8");
          let list = this.getApiByScript(content, componentDevUrl);
          apiList = apiList.concat(list);
        }
        obj.apis = this.mapArr(apiList);
      }
      allRouter.push(obj);
    });
    return allRouter;
  }

  // 路由中 解析所有api， 短地址与目标地址映射关系， 当做字典使用
  getApiList(filePath) {
    this.getApiUrl(filePath, (res) => {
      // 1. 正则匹配出键值对关系
      let baseList = fileContent.match(
        /(['"]{1}[a-zA-Z.]+['"]{1}\s?[:]+\s?['"]{1}[@a-zA-Z.\/{}]+['"]{1}),?/g
      );
      if(!baseList) return
      baseList.forEach((item) => {
        let obj = item.split(":");
        let key = obj[0].trim();
        let value = obj[1].trim();
        let str = /['"]{1}\@.*/;
        if (str.test(value)) {
          baseApi[this.formatUrl(key)] = this.formatUrl(value);
        }
      });
    });
  }

  // 读取相关带api的文件， 储存在变量中 ，  当做字典使用
  getApiUrl(inputPath, callBack) {
    let files = fs.readdirSync(inputPath);
    files.forEach((file) => {
      let filePath = this.formatPath(inputPath + "/" + file);
      let fileState = fs.statSync(filePath);
      if (fileState.isDirectory()) {
        // 如果是目录 递归
        this.getApiUrl(filePath);
      } else {
        let str = /(.*?\.js)/;
        if (!str.test(filePath)) return;
        this.options.moduleConfig.IMPORT_MODULES.forEach((moduleName) => {
          // 正则判断整合的映射api 是不是依赖模块下的
          var reg = new RegExp(`${moduleName}\/api\/[A-Za-z_-]*.js$`);
          let res = reg.test(filePath);
          if (res) {
            const data = fs.readFileSync(filePath, "utf-8");
            let b = data.replace("export default", "");
            fileContent += b;
          }
        });
      }
    });
    callBack && callBack.call();
  }

  // 路由中获取api， 进行转义 （取自动成router.js 文件遍历寻找）
  getApiByRouter(list) {
    let routerList = [];
    list.forEach((item) => {
      let url = baseApi[item[1]];
      let index = 0;
      let key = this.getApiKey(url); // 获取@前缀项目的key
      if (url) {
        index = url.indexOf(key+"/") + key.length;
        routerList.push([key, url ? url.substr(index + 1) : "", item[0]]);
      }
    });
    return routerList;
  }

  // 获取 api 前缀 （router 中获取）
  getApiKey(url) {
    if (!url) return;
    const reg = /(^@.+?\/)/g;
    if (!reg.test(url)) return;
    const replaceStr = RegExp.$1;
    const apiKey = replaceStr.substring(1, replaceStr.length - 1);
    return apiKey;
  }

  // script获取 api （取自动成router.js 文件遍历寻找）
  getApiByScript(content, srcUrl) {
    const component = compiler.parseComponent(content)
    if(!(component.script && component.script.content)) return;
    content = this.parseRoute(component.script.content)  // 取出组件中 script 内容解析
    let match = content.match(/(ask(\.[a-zA-Z.]*){1,10})[('")]/g);
    let subApi = [];
    if (match) {
      // TODO: 去重处理
      let list = Array.from(new Set(match));
      list.forEach((item) => {
        let apiObj = api;
        let codeList = item.replace(/"/g,"").replace(/'/g,"").split(".");
        for(let i = 1; i<codeList.length; i++) {
          if (apiObj) apiObj = this.getObjct(apiObj, codeList[i]);
        }
        if (!(apiObj && apiObj.url)) return;
        let index = apiObj.url.indexOf("/"); // 取出服务名
        let url = apiObj.url.substr(index + 1); // 具体api 地址
        if (url) {
          subApi.push([apiObj.url.substring(0, index), url, apiObj.type]);
        }
      });
    }
    return subApi;
  }

  getObjct(obj, code) {
    return obj[code] ? obj[code] : null;
  }

  // 一个地址去api重复 请求类型 + 请求地址 判断重复
  mapArr(list) {
    let newList = [];
    list.forEach((item) => {
      let index = newList.findIndex(it => it[1] === item[1] && it[2] === item[2]
      );
      if (index == -1) {
        newList.push(item);
      }
    });
    return newList;
  }
  /**
    * 路径统一
    */
  formatPath(p) {
    if (p && typeof p === "string") {
      const sep = path.sep;
      if (sep === "/") return p;
      else {
        return p.replace(/\\/g, "/");
      }
    }
    return p;
  }

  // script 内容中去除 $$route 内容
  parseRoute(txt) {
    let data = txt;
    let mark = '$$route:';
    let chart1 = '{';
    let chart2 = '}';
    let startIndex = data.indexOf(mark);
    let count = 0;
    let begingIndex;
    let endingIndex;
    if (startIndex === -1) {
      return '';
    }
    for (let i = startIndex; i < data.length; i++) {
      let w = data.charAt(i);
      if (w === chart1) {
        if (count === 0) begingIndex = i;
        ++count;
      } else if (w === chart2) {
        --count;
        if (count === 0) {
          endingIndex = i + 1;
          break;
        }
      }
    }
    return endingIndex ? data.slice(0, startIndex) + data.slice(endingIndex + 1) : '';
  }

  // 去除 空格 , ' " 
  formatUrl (url) {
    if (url) return url.replace(new RegExp(/['",\s]/, 'g'), '')
  }
}

module.exports = analysisPlugin;
